# -*- coding: utf-8 -*-

import re
import sys

class CommentFileInput:

    def __init__(self,_pathname,_inline_comment='#'):
        self.lno=self.last_readline_lineno=0
        self.eof_=False
        if sys.version_info[0]==2:
            self.fileobj=open(_pathname,'rU')
        else:
            self.fileobj=open(_pathname,mode='rU',encoding='utf-8')
        self.inline_comment_regexp='('+_inline_comment+'.*)'
        self.next_line=self.read_logical_line() 
        self.eof_exception="Read past end of file"
        
    def read_physical_line(self):
        line=self.fileobj.readline()
        if line:
            self.lno=self.lno+1
        else:
            self.eof_=True
        return line
        
    def lineno(self):
        return self.last_readline_lineno
        
    def eof(self):
        return self.eof_
        
    def read_logical_line(self):
        while 1:
            strippedline=re.sub(self.inline_comment_regexp,'',self.read_physical_line().rstrip('\n'))
            if self.eof():
                return None
            if strippedline:
                return strippedline
            
    # Reads and returns next non-empty line. On eof, returns None        
    def readline(self):
        retval=self.next_line
        self.last_readline_lineno=self.lno
        self.next_line=self.read_logical_line()
        return retval
        
    # Sets the exception to be used by readlinex()    
    def set_eof_exception(self,e):
        self.eof_exception=e
        
    # Reads and returns next non-empty line. On eof, throws exception.    
    def readlinex(self):
        if self.eof():
            raise self.eof_exception
        return self.readline()
        
