# -*- coding: utf-8 -*-

import sys

try:
    from ConfigParser import SafeConfigParser, NoOptionError, NoSectionError
except ImportError:
    from configparser import SafeConfigParser, NoOptionError, NoSectionError

DEFAULT_SECTION='SYSTEM'
VALID_SECTIONS=(DEFAULT_SECTION, 'SYSTEM', 'USER', 'MSG')

DEFAULT_CONFIGURATION='''\
[SYSTEM]
dict_atari_F=25

[USER]
showarea_fontsize=56
# width in character
showarea_width=36
wordadder_width=72

[MSG]
add_word_button_text=新言葉 +✍+
quit_button_text=終わり
save_dict_button_text=SAVE
save_dict_as_button_text=SAVE TO
load_dict_button_text=LOAD
merge_dict_button_text=ADDDICT
next_challenge_text=　　次　　➵　　　
edit_challenge_text=EDIT
save_edited_challenge_text=
cancel_edit_challenge_text=CANCEL
choose_dict_window_title=Welches Schweinderl hätten S'denn gern?
add_button_text=ADD
clear_button_text=CLEAR
oracle_label_text=OK? 正しいかい？
msg__must_not_be_empty=Feld darf nicht leer sein.
msg__must_not_contain_these_characters=Folgende Zeichen dürfen nicht verwendet werden:
msg__border_must_not_contain_these_characters=Folgende Zeichen dürfen nicht als erstes oder letztes nicht-leeres Zeichen verwendet werden:
msg__already_in_goi_pool=Einen Eintrag in dieser Schreibweise gibt es schon.
msg__word_added_confirmation=Der neue Ausdruck ist im Wörterbuch aufgenommen worden.
msg__dict_saved=Das Wörterbuch ist gesichert.
msg__dict_not_saved=Aufgrund eines Unbills ist das Wörterbuch nicht gesichert worden.
msg__dict_loaded=Das Wörterbuch ward geladen.
msg__dict_merged=Das Wörterbuch ward hinzugefügt.
msg__import_error=Fehler beim Laden des Wörterbuchs.
fmtmsg__showarea_has_unsaved_changes=Du hast den Text geändert auf:
                                     %%s
                                     Willst Du diese Änderung übernehmen?
msg__confirm_cancel_edit=Willst Du wirklich, daß Deine sicher sehr wertvollen und wohlüberlegten Änderungen in diesem Feld verloren gehen?
msg__no_suitable_word=Kein passendes Wort (mehr) vorhanden
msg__entry_already_exists=Dieser Eintrag existiert bereits im Wörterbuch:

'''

def so(v1,v2=None):
    if v2 is None:
        section=DEFAULT_SECTION
        option=v1
    else:
        section=v1
        option=v2
    return (section,option)
        
class StringAdapter(object):
    '''Makes a string object be useable for ConfigParser. Only readline() is needed'''

    def __init__(self,_s):
        # Need to include \n in the result
        self.s=str(_s).splitlines(True)
        self.current=0
        
    def readline(self):
        if len(self.s) <= self.current:
            result=''
        else:
            result=self.s[self.current]
            self.current=self.current+1
        return result

# Adapting StringAdapter for Python 3

def readline_generator(obj):
    line=obj.readline()
    while line:
        yield line
        line=obj.readline()

def make_string_adapter(_filename):
    sa=StringAdapter(_filename)
    if sys.version_info[0]==2:
      sa_arg=sa
    else:
      sa_arg=readline_generator(sa)
    return sa_arg        
    
class ConfigINI(object):
    
    def __init__(self,_filename=None):
        self.parser=SafeConfigParser()
        if _filename is None:
            self.__init__(DEFAULT_CONFIGURATION)
        elif '\n' in _filename and len(_filename) > 0:
            # The filename is not a filename, but a string containing the configuration. 
            self.parser.readfp(make_string_adapter(_filename),'<string>')
        else:
            # For the time being, it is OK to throw an exception if we have a problem with the file
            self.parser.read(_filename) 
            
    def o(self,v1,v2=None):
        '''o(SECTION,OPTION) returns the value of OPTION within SECTION
           o(OPTION) returns the value of OPTION within the DEFAULT_SECTION
           o('',OPTION) returns the value of OPTION within any section. If OPTION occurs in more
                        than one section, one of the values is return. It is unspecified, which one.
        '''
        (sec,opt)=so(v1,v2)
        result=None
        if sec:
            try:
                result=self.parser.get(sec,opt)
                # ConfigParser removes leading and trailing space from values. By definition, we consider
                # the comment character (#) as a quotation character and allow embedding spaces. This 
                # means that a '#' can not occur as the first AND last character of a value.
                if result is not None and result[0] == '#' and result[-1] == '#' and len(result) > 2:
                    result=result[1:-1]
            except NoOptionError:
                pass # Not treated as error
            except NoSectionError:
                # Not an error if it is a permitted section
                if not sec in VALID_SECTIONS:
                    raise
        else:
            for sec in VALID_SECTIONS:
                result=self.o(sec,opt)
                if result is not None:
                    break
        return result
        
config_default_ini=ConfigINI(DEFAULT_CONFIGURATION)
section_for_cache={}
def section_for(option):
    result=section_for_cache.get(option,None)
    if result is None:
        for sec in VALID_SECTIONS:
            if config_default_ini.o(sec,option) is not None:
                if result is None or result == sec:
                    result=sec
                else:
                    raise Exception('Option '+option+' defined twice in default configuration: '+result+' and '+sec)
        if result is not None:
            section_for_cache[option]=result
    return result
    
