#!/usr/bin/python

from inspect import currentframe,getouterframes,stack
from os.path import basename,splitext

from kanren_util import *

def _format_frameinfo(frameinfo):
    file,line,fct=frameinfo[1:4]
    file=splitext(basename(file))[0]
    return file+'('+str(line)+')'+fct
    
class _Dbg: # Leading underscore means: Won't be imported

    def __init__(self,logfile=None):
        self.handle=None
        if logfile is not None:
            try:
                self.handle=uopen(logfile,'w')
            except IOError:
                e=sys.exc_info()[1]
                showwarning("Access problem","Can not create logfile. "+e.strerror)
        
    def dbg(self,frameindex,logtype,*_info):
        text=" ".join([logtype,_format_frameinfo(stack()[frameindex])+':']+list(map(str,_info)))
        if self.handle:
            self.handle.write(text+"\n")
        else:
            prnt(text)

_default_debug_object=None
_default_debug_filename=None # Logging goes to stdout unless this is set

def set_default_debug_file(filename):
    global _default_debug_filename
    global _default_debug_object
    _default_debug_filename=filename
    _default_debug_object=None # delay creation until first log event
    
def debug_setup(filename=None):
    global _default_debug_object
    _default_debug_object=_Dbg(filename)
    
def _dbg(logtype,*_info):
    global _default_debug_filename
    global _default_debug_object
    if not _default_debug_object:
        debug_setup(_default_debug_filename)
    _default_debug_object.dbg(3,logtype,*_info)
    
def _callchain(fromframe=3,toframe=SEE_BELOW):
    if toframe is SEE_BELOW:
        toframe=fromframe+2
    s=stack()
    if toframe >= len(s):
        toframe=len(s)-1
    result=''
    for frameindex in range(fromframe,toframe+1):
        result = result + ' from ' + _format_frameinfo(s[frameindex])
    return result
    
def log(*_info):
    _dbg('DEBUG',*_info)
    
def todo(*_info):
    _dbg('TODO ',*_info)
    
def bark(*_info):
    _dbg('DEBUG',_callchain(),*_info)
    
def fatal(*_info):
    _dbg('FATAL',*_info)
    
