# -*- coding: utf-8 -*-

from kanren_cf import ConfigINI, section_for
from kanren_dict import *

class Settings:

    def __init__(self):
        self.spath=[ConfigINI()] # Default Settings
        self.donotaskagainsteps=3 # Default
        
    def addcf(self,_file_or_cfstring):
        self.spath.insert(0,ConfigINI(_file_or_cfstring))
        
    def o(self,v1,v2=None):
        result=None
        for cf in self.spath:
            result=cf.o(v1,v2)
            if result is not None:
                break
        return result

    # Assume that opt is defined in the same section where it is in the default configuration
    def lookup(self,opt):
        return self.o(section_for(opt),opt)
        
    # If settings.x() is invoked, and x is not defined, a function for option 'x' is returned.
    def __getattr__(self,name):
        value=self.lookup(name)
        return lambda result=value: result
        
    def display_kind_name(self,kind):
        if kind == GAIKOKUJI:
            return "Deutsch"
        else:
            return KIND_NAME[kind]
        
    def start_button_text(self,kind):
        texts=['始めましょう','はじめましょう','START']
        return texts[kind]
        
    def save_edited_challenge_text(self):
        return self.save_dict_button_text()
        
    def add_word_window_title(self):
        return self.add_word_button_text()
        
    def do_not_ask_again_steps(self):
        return self.donotaskagainsteps
        
    def set_do_not_ask_again_steps(self,_new_value):
        self.donotaskagainsteps=_new_value
        return self
 
    def ok_label(self):
        return "☑ (はい)" # or: ✓
        
    def notok_label(self):
        return "☒ (いいえ)" # or: ✗
      
    def fmtmsg__showarea_has_deleted_text(self):
        return "Du hast die %s-Definition gelöscht. Willst Du diese Änderung übernehmen?"

    def fmtmsg__level_changed_to(self):
        return "Wort wurde auf Level %s verschoben"
        
        
    # Configuration of the X, Y and F parameters for the hit count adjustment (see kanren_dict.py)
    def dict_atari_X(self):
        return 3
        
    def dict_atari_Y(self):
        return self.dict_atari_X()

def settings():
    return Settings()
