#!/usr/bin/python
# -*- coding: utf-8 -*-

# Widget with one line of text, and button for going to the next word, and editing

try:
  from Tkinter import * # Python 2
  from tkFont import Font
  from tkMessageBox import askyesno,showerror
except ImportError:
  from tkinter import * # Python 3
  from tkinter.font import Font
  from tkinter.messagebox import askyesno,showerror

import kanren_settings
from kanren_dict import *
from kanren_util import *
from kanren_varlabel import *

class Showarea(object):

    def __init__(self,_parent,_settings,initial_text,_kind,_goidict):
        self.S=_settings
        self.D=_goidict
        self.kind=_kind
        self.parent=_parent
        self.widget=Frame(_parent)
        self.control=Frame(self.widget)
        self.hscrollbar=Scrollbar(self.control,orient=HORIZONTAL,command=self.scrollHandler)
        self.text_v=StringVar() # the Tk variable holding the text to be displayed
        self.textarea=Entry(self.widget, textvariable=self.text_v, font=Font(size=self.S.showarea_fontsize()), state="readonly", width=_settings.showarea_width(), borderwidth=6, relief=GROOVE)
        self.text_v.set(initial_text)
        self.textarea.pack()
        self.buttons=Frame(self.control)
        self.button_cancel_edit=Button(self.buttons,text=self.S.cancel_edit_challenge_text(),state=DISABLED,command=self.cancel_edit)
        self.button_cancel_edit.pack(in_=self.buttons,side=LEFT)
        self.show_dirty_field=VarLabel(self.buttons,' ',borderwidth=0,width=1)
        self.show_dirty_field.pack(in_=self.buttons,side=LEFT)
        self.button_edit_challenge=Button(self.buttons,text=self.S.edit_challenge_text(),state=DISABLED,command=self.edit_challenge)
        self.button_edit_challenge.pack(in_=self.buttons,side=LEFT)
        self.buttons.pack(side=RIGHT)
        self.hscrollbar.pack(side=LEFT,fill="x",expand=True)
        self.control.pack(fill="both",expand=True)
        self.hide_cancel_edit()
        self.textarea['xscrollcommand']=self.hscrollbar.set
        self.set_dirty_to(False)
        # http://infohost.nmt.edu/tcc/help/pubs/tkinter/web/entry-scrolling.html
        
    def scrollHandler(self,*args):
        op,how_many=args[0],args[1]
        if op == 'scroll': # clicking the scroll bar
            units=args[2] # This parameter is not set for moveto 
            self.textarea.xview_scroll(how_many, units)
        elif op == 'moveto': # dragging the scroll bar
            self.textarea.xview_moveto(how_many)
        return self
        
    def editing_is_enabled(self):
        # Clicking on EDIT turns on editing. Hence if the text of the button is still 'EDIT', we are
        # not in edit mode (yet). Maybe we should instead check the state of the textarea (NORMAL/
        # READONLY)? 
        return self.button_edit_challenge['text'] != self.S.edit_challenge_text()
        
    def edit_challenge(self):
        if self.editing_is_enabled():
            # SAVE has been clicked
            self.save_edit()
        else:
            # Start editing the text
            self.textarea['state']=NORMAL
            self.text_before_editing=self.gettext()
            self.show_cancel_edit()
            self.button_edit_challenge['text']=self.S.save_edited_challenge_text()
            self.textarea.focus() # Give the focus to the entry field, so that the user can start typing
        return self
        
    def save_edit(self):    
        newtext=whitespace_normalize_word(self.gettext())
        if not self.editing_is_enabled():
            raise Exception("Plausibility check violated: Trying to save showarea field, where editing is not enabled.\nNew text="+newtext)
        self.set_dirty_to(self.text_before_editing != newtext)
        if self.is_dirty():
            self.settext(newtext) # Because normalization might have edited it
            verify_errmsg=self.D.verify_word(newtext,self.kind) # Check syntax
            if len(verify_errmsg) > 0:
                showerror("ERROR",verify_errmsg)
                return self # stay in old state
            if self.D.already_exists(self.kind,newtext): # Don't allow duplicates
                showerror("ERROR",self.S.msg__already_in_goi_pool())
                return self # stay in old state
        self.textarea['state']='readonly'
        self.hide_cancel_edit()
        self.button_edit_challenge['text']=self.S.edit_challenge_text()
        return self
        
    def cancel_edit(self,force=False):
        if not self.editing_is_enabled():
            raise Exception("Plausibility check violated: Trying to cancel editing of showarea field, where editing is not enabled.")
        if force or not self.text_has_been_changed() or askyesno("Confirm Cancel",self.S.msg__confirm_cancel_edit()):
            self.set_dirty_to(False)
            self.settext(self.text_before_editing)
            self.hide_cancel_edit()
            self.button_edit_challenge['text']=self.S.edit_challenge_text()
        return self
        
    def has_unsaved_edits(self):
        return self.editing_is_enabled() and self.text_has_been_changed()
        
    def text_has_been_changed(self):
        return self.text_before_editing != whitespace_normalize_word(self.gettext())
        
    def hide_cancel_edit(self):
        show_widget_disabled(self.button_cancel_edit)
        
    def show_cancel_edit(self):
        show_widget_enabled(self.button_cancel_edit)
        
    def pack(self):
        self.widget.pack()

    def enable_edit(self):
        show_widget_enabled(self.button_edit_challenge)
        
    def disable_edit(self):
        show_widget_disabled(self.button_edit_challenge)
        
    def settext(self,newtext):
        self.text_v.set(newtext)
        self.widget.update_idletasks()
        return self

    def gettext(self):
        return tkinter_to_str(self.text_v.get())
        
    def is_dirty(self):
        return self.dirty

    def set_dirty_to(self,_flag):
        self.dirty=_flag
        if _flag:
            dirty_indicator='*'
        else:
            dirty_indicator=' '
        self.show_dirty_field.settext(dirty_indicator)
        
