#!/usr/bin/python
# -*- coding: utf-8 -*-

# NOTE: No logging in this module (because needed by log module)

import sys

SEE_BELOW=object() # For modelling default arguments to functions, which depend on other args of the same fct
# Usage: def f(x,y=SEE_BELOW):
#          if y is SEE_BELOW:
#            y=2*x
# Often, we can not use y=None, because None might be a valid argument.
# Idea stolen from NPE at Stackoverflow.

def _set_widget_state(widget,newvalue,show):
    oldvalue=widget['state']
    widget['state']=newvalue
    if show:
        widget.update_idletasks()
    return oldvalue
    
def show_widget_enabled(widget):
    return _set_widget_state(widget,'normal',True)
    
def show_widget_disabled(widget):
    return _set_widget_state(widget,'disabled',True)

def bool_pp(val,name='true'):
    if val:
        return name
    else:
        return 'not '+name

# Functions for dealing with Python2/Python3 incompatibilities

def prnt(*a):
    '''Replacement for print, to cope with incompatibilities between Python 2 and Python 3.
       prnt(a,b,c) behaves under Python 2 like print a,b,c and under Python 3 like print(a,b,c).
       Note that print(a,b,c) is syntactically valid in Python 2 too, but has a different effect
       from print a,b,c
    '''
    # print(' '.join(map(str,a))) # Crashes sometimes on Windows with Python 2
    print(' '.join(map(ucguard_str,a)))

if sys.version_info[0]==2:

  def tkinter_to_str(v):
    '''In Python 2, Tk strings (for example from a StringVar) need to be explicitly converted to
       Unicode strings. In Python 3, they are already in the right format, and using encode() would
       turn them into a string of bytes. 
       Solution: Replace v.encode('utf-8') by tkinter_to_str(v)
    '''
    return v.encode('utf-8')    
    
  def ucguard(v):
    '''Similar to tkinter_to_str, but v can be any type.
       Non-string-types are returned unchanged.
       Note: Some functions in tkinter (for example asksaveasfilename) return in some 
             implementations a byte string (str), and in others a unicode, and this causes a
             problem when catenated with other byte strings.
    '''
    if type(v).__name__ == 'unicode':
        return v.encode('utf-8')
    else:
        return v
        
  def uopen(path,mode):
        return open(path,mode)
        
else: # In Python 3 we have only one string type (str), which corresponds to unicode in Python 2

  def tkinter_to_str(v):
    return v
        
  def ucguard(v):
    return v
    
  def uopen(path,mode):
    return open(path,mode=mode,encoding='utf-8')
        
def ucguard_str(v):
    return str(ucguard(v))
    
    
    
