#!/usr/bin/python
# -*- coding: utf-8 -*-

# Widget for adding a word to the database

try:
  from Tkinter import *
  from tkFont import Font
except ImportError:
  from tkinter import *
  from tkinter.font import Font

from re import split

import kanren_settings
from kanren_dict import *
from kanren_util import tkinter_to_str
from kanren_varlabel import *

class Wordadder:
    '''Instances of this class are windows, which allow to dynamically add a new word to the 
       dictionary. The window is invisible after creation, and needs to be explicitly set to visible
       using the show() function.
    '''
    
    # _parent : Parent widget
    # _settings : Object of type kanren_settings.Settings
    # _goidict : Object of type kanren_dict.GoiDict
    # _fetch_last_asked_event_number_cb : Callback which fetches event number to store with the new word
    def __init__(self,_parent,_settings,_goidict,_fetch_last_asked_event_number_cb):
        self.parent=_parent
        self.S=_settings
        self.D=_goidict
        self.shown=False # Are we visible?
        self.flaenc=_fetch_last_asked_event_number_cb
        
    def show(self):
        '''Make the Wordadder visible, if it isn't already'''
        if not self.shown:
            self.widget=Toplevel(self.parent)
            parent_geom_arr=split(r'\D',self.parent.geometry()) # NNNxMMM+x+y
            self.widget.geometry(("+%d+" % (int(parent_geom_arr[2])/2))+parent_geom_arr[3])
            self.widget.title(self.S.add_word_window_title())
            self.widget.protocol('WM_DELETE_WINDOW',self.unshow)
            # We use the grid manager instead of the packer, since it is simpler. 
            # Note: NEVER use both managers within the same master window!!!
            # Layout:
            #   TEXT    ENTRY       ADDBUTTON
            #   TEXT    ENTRY
            #   TEXT    ENTRY
            #   STATUSLINE          CLEARBUTTON
            self.word_entry_field=new_repr_list()
            for i in ALLREP_RANGE:
                Label(self.widget,text=self.S.display_kind_name(i)).grid(row=i,column=0)
                # The width attribute is the number of characters, for calculating the size. The
                # field is scrollable, and the text can be any length.
                # TODO: Calculate the width in pixel from the screen geometry.
                self.word_entry_field[i]=Entry(self.widget,width=self.S.wordadder_width()) 
                self.word_entry_field[i].grid(row=i,column=1)
            self.statusline=VarLabel(self.widget," "*24,borderwidth=0,relief=FLAT,width="80")
            self.statusline.grid(row=REPRESENTATIONS,column=0,columnspan=2)
            self.add_button=Button(self.widget,text=self.S.add_button_text(),command=self.add_word,height=2)
            self.add_button.grid(row=0,column=2,rowspan=REPRESENTATIONS-1,sticky=W+E)    
            self.clear_button=Button(self.widget,text=self.S.clear_button_text(),command=self.clear_fields)
            self.clear_button.grid(row=REPRESENTATIONS,column=2)
            self.widget.update_idletasks()
            self.shown=True
        
    def unshow(self):
        if self.shown:
            self.widget.destroy()
            self.shown=False
            
    def set_statusline_kind(self,kind,msg):
        self.statusline.settext(KIND_NAME[kind]+': '+msg)
        
    def add_word(self):
        # Verify input for correctness. 
        text=new_repr_list()
        for i in ALLREP_RANGE:
            text[i]=t=whitespace_normalize_word(tkinter_to_str(self.word_entry_field[i].get())) # trim string
            # Verify that the fields are filled. 
            errmsg=self.D.verify_word(t,i)
            if len(errmsg) > 0:
                self.set_statusline_kind(i,errmsg)
                return False
        self.D.add(Goi(self.flaenc(),text[KANJI],text[KANA],text[GAIKOKUJI],1))  
        self.clear_fields()
        self.statusline.settext(self.S.msg__word_added_confirmation())
        self.D.set_dirty()
        
    def clear_fields(self):
        self.statusline.clear()
        for i in ALLREP_RANGE:
            self.word_entry_field[i].delete(0,END)
